︠14a79ae7-8c0d-4006-9bf6-8744818cbf25s︠
import numpy
from math import isnan

def findFunctionPoints (f, filter, steps=5):
    """
    This creates and returns an array of values for a function based on the value of the independent variable
    The points are created between the values 'xmin' and 'xmax'. This is called the domain of the independent variable
    The points are chosen so that there are 'steps' number of intervals (spaces between two points) across the domain.
    We multiply xmax by 1 + 1/steps to create a point that is one step beyond the range.
    The array returnd is a one dimensional array of two dimensional tuples (x,f(x))
    """
    points = []
    global domMin, domMax, inverseOn, inverseSaveDom, aspectRatio
    if inverseOn:
        xmin, xmax = inverseSaveDom
    inverseOn = False
    for i in numpy.arange(domMin, domMax*(1+2/steps), (domMax - domMin)/steps):
        try:
            f(i)
        except ValueError:
            i=i-.00001
        if f(i) in RR and not isnan(f(i)):
            abscissa = i
            if filter == 'normal':
                ordinate = f(i)
            elif filter == 'input negative':
                ordinate = f(-i)
            elif filter == 'negative':
                ordinate = -f(i)
            elif filter == 'absolute value':
                ordinate = abs(f(i))
            elif filter == 'square':
                ordinate = f(i)^2
            elif filter =='inverse':
                inverseOn = True
                inverseSaveDom = (domMin, domMax)
                ordinate = i
                abscissa = f(i)
            points.append((abscissa, ordinate))
#             if abscissa > domMax:
#                 domMax = abscissa
#             if abscissa < domMin:
#                 domMin = abscissa
        else:
            continue
    return points

def createSlopeFunction (points):
    """
    This creates a points array of the slope between two points and assigns that value as an estimate of the slope of the curve at the higher valued point.
    The array returned is a one dimensional array of two dimensional tuples (x,slope at x)
    """
    global domMin, domMax
    newPoints = []
    slopeFunctionGraph = plot(0)
    first = True
    for firstPoint in points:
        if first:
            lastPoint = firstPoint
            first = False
            continue
        slope = (firstPoint[1]-lastPoint[1])/(firstPoint[0]-lastPoint[0])
        lastPoint = firstPoint
        newPoints.append((firstPoint[0],slope))
    return newPoints


def plotPoints (points,ymin,ymax):
    """
    This function takes a points array and plots it on a graph
    It returns the graph
    """
    global domMin, domMax
    Graph = plot(0)
    strongGraph = plot(0)
    pointsGraph = []
    markerColor = 'deepskyblue'
    for p in points:
        Graph += point(p,xmin=domMin,xmax=domMax,size=150,color=markerColor,marker='h',alpha=0.4)
        strongGraph = point(p,xmin=domMin,xmax=domMax,size=600,color=markerColor,marker='h',alpha=1)
        pointsGraph.append(Graph+strongGraph)
    return pointsGraph

def showContinuity (points):
    """
    This function takes a points array and plots it as segments on a graph
    """
    global domMin, domMax
    thickness = 2
    color = 'maroon'
    segmentsGraph = line(points,xmin=domMin,xmax=domMax,thickness=thickness,color=color)
    return segmentsGraph


def plotSegments (points,ymin,ymax):
    """
    This function takes a points array and plots it as segments on a graph
    """
    global domMin, domMax
    segmentsGraph = line(points,xmin=domMin,xmax=domMax)
    color = 'maroon'
    thickness = 2
    Graph = plot(0,ymin=ymin,ymax=ymax)
    segmentsGraph = []
    first = True
    for firstPoint in points:
        if first:
            lastPoint = firstPoint
            first = False
            continue
        slope = (firstPoint[1]-lastPoint[1])/(firstPoint[0]-lastPoint[0])
        if slope <= 0:
            slopeColor = 'red'
        else:
            slopeColor = 'lime'
        rise = firstPoint[1]-lastPoint[1]
        run = firstPoint[0]-lastPoint[0]
        if firstPoint[1] > lastPoint[1]:
            thirdPoint = (firstPoint[0],lastPoint[1])
        else:
            thirdPoint = (lastPoint[0],firstPoint[1])
        g(x) = slope*(x-firstPoint[0]) + firstPoint[1]
        boldGraph = line([firstPoint, lastPoint],xmin=domMin,xmax=domMax,color=color,alpha=1,thickness=thickness+1,ymin=ymin,ymax=ymax)
        boldGraph += line([firstPoint,thirdPoint],xmin=domMin,xmax=domMax, color=slopeColor,alpha=1,linestyle=':',thickness=thickness,ymin=ymin,ymax=ymax)
        boldGraph += line([lastPoint,thirdPoint],xmin=domMin,xmax=domMax,color=slopeColor,alpha=1,linestyle=':',thickness=thickness,ymin=ymin,ymax=ymax)
        if firstPoint[1]<=ymax and firstPoint[1]>=ymin and lastPoint[1]<=ymax and lastPoint[1]>=ymin:
            if firstPoint[1] > lastPoint[1]:
                midPoint = (((firstPoint[0]+thirdPoint[0])/2),((firstPoint[1]+thirdPoint[1])/2))
                boldGraph += text('rise',midPoint,xmin=domMin,xmax=domMax,color='black',alpha=1,ymin=ymin,ymax=ymax)
                midPoint = (((lastPoint[0]+thirdPoint[0])/2),((lastPoint[1]+thirdPoint[1])/2))
                boldGraph += text('run',midPoint,xmin=domMin,xmax=domMax,color='black',alpha=1,ymin=ymin,ymax=ymax)
            else:
                midPoint = (((firstPoint[0]+thirdPoint[0])/2),((firstPoint[1]+thirdPoint[1])/2))
                boldGraph += text('run',midPoint,xmin=domMin,xmax=domMax,color='black',alpha=1,ymin=ymin,ymax=ymax)
                midPoint = (((lastPoint[0]+thirdPoint[0])/2),((lastPoint[1]+thirdPoint[1])/2))
                boldGraph += text('rise',midPoint,xmin=domMin,xmax=domMax,color='black',alpha=1,ymin=ymin,ymax=ymax)
#         show(boldGraph)
        Graph += line([firstPoint,lastPoint],xmin=domMin,xmax=domMax,color=color,alpha=0.5,thickness=thickness,ymin=ymin,ymax=ymax)
        segmentsGraph.append(Graph+boldGraph)
        lastPoint = firstPoint
    return segmentsGraph


def plotSlopes (points, shiftText,ymin,ymax):
    """
    This function calculates the slope between any two consecutive pounts in a points array.
    It then displays those points on a graph between the two points by calculating the midpoint and thnr shifting it down 1.5 pixels to make room for a line.
    It returns the graph
    """
    global domMin, domMax
    maxPlots = 30
    modBase = (len(points)+(50-1))//50
    Graph = plot(0)
    strongGraph = plot(0)
    slopeGraph = []
    #
    first = True
    for firstPoint in points:
        if first:
            lastPoint = firstPoint
            first = False
            continue
        slope = (firstPoint[1]-lastPoint[1])/(firstPoint[0]-lastPoint[0])
        color = 'red'
        if slope >= 0:
            color = 'lime'
        plotAt = (((firstPoint[0]+lastPoint[0])/2),((firstPoint[1]+lastPoint[1])/2)-shiftText)
        markerAt = (plotAt[0],slope)
        if plotAt[1]<=ymax and plotAt[1]>=ymin:
            Graph += point(markerAt,xmin=domMin,xmax=domMax,size=200,color=color,marker='*',alpha=0.5)
            strongGraph = point(markerAt,xmin=domMin,xmax=domMax,size=800,color=color,marker='*',alpha=1)
            Graph += text(str(slope.n(prec=4)),plotAt,xmin=domMin,xmax=domMax,color=color)
        slopeGraph.append(Graph+strongGraph)
        lastPoint = firstPoint
    return slopeGraph


def plotLines (points,ymin,ymax):
    """
    This function ceates a linear funtion of the line between any two consecutive pounts in a points array.
        1. canculate the slope between two points
        2. use the slope and one of the points to create the fucntion g(x)
    It then displays those lines on a graph between the two points.
    It returns the graph
    """
    global domMin, domMax
    thickness = 2
    color = 'chocolate'
    markerColor = 'deepskyblue'
    Graph = plot(0)
    lineGraph = []
    first = True
    for firstPoint in points:
        if first:
            lastPoint = firstPoint
            first = False
            continue
        slope = (firstPoint[1]-lastPoint[1])/(firstPoint[0]-lastPoint[0])
        g(x) = slope*(x-firstPoint[0]) + firstPoint[1]
        boldGraph = plot(g(x),xmin=domMin,xmax=domMax,color=color,alpha=1,thickness=thickness+1)
        boldGraph += point(firstPoint,xmin=domMin,xmax=domMax,size=600,color=markerColor,marker='h',alpha=1)
        boldGraph += point(lastPoint,xmin=domMin,xmax=domMax,size=600,color=markerColor,marker='h',alpha=1)
        Graph += plot(g(x),xmin=domMin,xmax=domMax,color=color,alpha=0.5,thickness=thickness)
        lineGraph.append(Graph+boldGraph)
        lastPoint = firstPoint
    return lineGraph

# Main Procedure

# Global Variables

domMin = -10
domMax = 10
xStep = 0.25
inverseOn = False
inverseSaveDom = (domMin, domMax)
Xrange = (domMin, domMax)
aspectRatio = 100
fileCount = 1

@interact (width=200,auto_update=False)
def _(  #f         = input_box      (default=((x+4)*(x+1)*(x-1)*(x-4)).expand(), label='function'),
        f         = selector       ([(x+4)*(x-5),
                                     -(x-3)(x+4)
                                     (x+4)*(x+1)*(x-5),
                                     (x+5)*(x+2)*(x-1)*(x-4),
                                     1/x,
                                     1/(x-3)+4,
                                     sqrt(x-3)+2,
                                     e^x,
                                     e^(x+2)-3,
                                     log(x),
                                     log(x+3),
                                     1-e^-x
                                    ],label = 'function',buttons=False, width=50),
        xrange    = range_slider   (round(domMin*2),round(domMax*2),xStep,label='xrange',default=(domMin/2, domMax/2),width = 200),
        yrange    = range_slider   (-400,500,10,label='yrange',default=(-50, 100),width = 200),
        type      = selector       (['table','points','lines','segments','plot'],label = 'type',buttons=true, width=20, default= 'table'),
        level     = selector       (['function','derivative','2nd derivative','continuity'],label = 'level',buttons=true, width=20, default= 'function'),
        tables    = selector       (['function','derivative','2nd derivative'],label = 'tables',buttons=false, width=20, default= 'function'),
        showSlope = checkbox       (label = 'Show Slopes', default= False),
        animateGr = checkbox       (label = 'Show Animation', default= True),
        logAnis   = checkbox       (label = 'Log Animations', default= True),
        filter    = selector       (['normal', 'negative','absolute value','square', 'inverse'],label = 'filter',buttons=true, width=20, default= 'normal'),
        shiftText = slider         (-50,30,1,label='shift',default=20,width = 20),
        aspect    = slider         (1,300,1,label='aspect',default=100,width = 20),
        deltaX    = slider         (5,100,1,label='deltaX',default=5,width = 20),
        fName     = input_box      (default="general quartic", type=str, label='Log File Name'),
         ):

    global fileCount, domMin, domMax, Xrange, aspectRatio
    f=f.expand()

    def displayOutput(Points, plotFunction):
        global fileCount
        graphs = plotFunction(Points,ymin,ymax)
        slopeGraphs = plotSlopes(Points, -shiftText,ymin,ymax)
        if not animateGr:
            thePlot = graphs[len(graphs)-1]
            if showSlope:
                thePlot += slopeGraphs[len(slopeGraphs)-1]
            thePlot.show(figsize=[10,6],xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax)
        else:
            if showSlope:
                for i in range(len(slopeGraphs)):
                    graphs[i] += slopeGraphs[i]
                graphs[len(graphs)-1] += slopeGraphs[len(slopeGraphs)-1]
            show(animate(graphs,figsize=[10,6],xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax,gridlines='major',gridlinesstyle=dict(color="slategray", linestyle=":")),delay=500/deltaX)
            if logAnis:
                a=animate(graphs,figsize=[10,6],xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax,gridlines='major',gridlinesstyle=dict(color="slategray", linestyle=":"))
                a.gif(savefile = 'graph log files/'+fName+str(fileCount)+'.gif',delay=500/deltaX,iterations=2)
                fileCount += 1

    domMin, domMax = xrange
    ymin, ymax = yrange
    deltaXRange = 1
    aspectRatio = aspect
    functionPoints = findFunctionPoints(f, filter=filter, steps=deltaX)
    slopeFunctionPoints = createSlopeFunction(functionPoints)
    secondSlopeFunctionPoints = createSlopeFunction(slopeFunctionPoints)
    html('<h1 align=center>Function Plotting Toolkit</h1>')
    if filter == 'normal':
        show(f)
    elif filter == 'negative input':
        show(f(-x))
    elif filter == 'negative input':
        show(-f)
    elif filter == 'absolute value':
        show(abs(f))
    elif filter == 'square':
        show(f^2)
    elif filter =='inverse':
        show(f)
    if level == 'function':
        if type == 'plot':
            if filter == 'normal':
                showPlot = plot(f(x),figsize=[10,6],xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax,gridlines='major',gridlinesstyle=dict(color="slategray", linestyle=":"))
                showPlot.show()
                showPlot.save('graph log files/'+fName+str(fileCount)+'.pdf')
                fileCount += 1
            elif filter == 'negative':
                plot(-f(x) ,figsize=[10,6],xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax,gridlines='major',gridlinesstyle=dict(color="slategray", linestyle=":")).show()
            elif filter == 'absolute value':
                plot(abs(f(x)),figsize=[10,6],xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax,gridlines='major',gridlinesstyle=dict(color="slategray", linestyle=":")).show()
            elif filter == 'square':
                plot(f(x)^2,figsize=[10,6],xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax,gridlines='major',gridlinesstyle=dict(color="slategray", linestyle=":")).show()
            elif filter =='inverse':
                plot(f(x),figsize=[10,6],xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax,gridlines='major',gridlinesstyle=dict(color="slategray", linestyle=":")).show()
        elif type == 'segments':
            displayOutput(functionPoints, plotSegments)
        elif type == 'points':
            displayOutput(functionPoints, plotPoints)
        elif type == 'lines':
            displayOutput(functionPoints, plotLines)
        elif type == 'table':
            if tables == 'function':
                tempTable = []
                for row in range(0,len(functionPoints)):
                    tempTable.append([round(functionPoints[row][0],2), round(functionPoints[row][1],2)])
                pointsTable = table(tempTable,header_row=['$x$',"$y=f(x)$",],frame=True, align='right')
                tempTable.insert(0,['$x$',"$y=f(x)$"])
            if tables == 'derivative':
                tempTable = []
                for row in range(0,len(slopeFunctionPoints)):
                    tempTable.append([round(functionPoints[row+1][0],2), round(functionPoints[row+1][1],2), round(slopeFunctionPoints[row][1],2)])
                pointsTable = table(tempTable,header_row=['$x$',"$y=f(x)$", "$y=f'(x)$"],frame=True, align='right')
                tempTable.insert(0,['$x$',"$y=f(x)$", "$y=f'(x)$"])
            if tables == '2nd derivative':
                tempTable = []
                for row in range(0,len(secondSlopeFunctionPoints)):
                    tempTable.append([round(functionPoints[row+2][0],2), round(functionPoints[row+2][1],2) , round(slopeFunctionPoints[row+1][1],2), round(secondSlopeFunctionPoints[row][1],2)])
                pointsTable = table(tempTable,header_row=['$x$',"$y=f(x)$", "$y=f'(x)$", "$y=f''(x)$"], frame=True, align='right')
                tempTable.insert(0,['$x$',"$y=f(x)$", "$y=f'(x)$", "$y=f''(x)$"])
            try:
                show(pointsTable)
                openFile = open('graph log files/'+fName+str(fileCount)+'.txt', 'w')
                openFile.write(str(pointsTable))
                fileCount += 1
            except Exception as errmsg:
                print('table print error')
                html.table(rows=tempTable,header=True)
    elif level == 'derivative':
        pretty_print('derivative',f.derivative(x))
        if type == 'plot':
            plot(f(x).derivative(x) ,figsize=[10,6],xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax,gridlines='major',gridlinesstyle=dict(color="slategray", linestyle=":")).show()
        elif type == 'segments':
            displayOutput(slopeFunctionPoints, plotSegments)
        elif type == 'points':
            displayOutput(slopeFunctionPoints, plotPoints)
        elif type == 'lines':
            displayOutput(slopeFunctionPoints, plotLines)
        elif type == 'table':
            try:
                show((table([(n(i[0],prec=6),n(i[1],prec=15)) for i in slopeFunctionPoints],header_row=['$x$',"$y=f'(x)$"],frame=True, align='right')))
            except Exception as errmsg:
                tempTable = ([(n(i[0],prec=6),n(i[1],prec=15)) for i in secondSlopeFunctionPoints])
                tempTable.insert(0,['$x$',"$y=f(x)$"])
                html.table(rows=tempTable,header=True)
    elif level ==  '2nd derivative':
        pretty_print('derivative',f.derivative(x))
        pretty_print('2nd derivative', f.derivative(x).derivative(x))
        if type == 'plot':
            plot(f(x).derivative(x).derivative(x) ,figsize=[10,6],xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax,gridlines='major',gridlinesstyle=dict(color="slategray", linestyle=":")).show()
        elif type == 'segments':
            displayOutput(secondSlopeFunctionPoints, plotSegments)
        elif type == 'points':
            displayOutput(secondSlopeFunctionPoints, plotPoints)
        elif type == 'lines':
            displayOutput(secondSlopeFunctionPoints, plotLines)
        elif type == 'table':
            try:
                show((table([(n(i[0],prec=6),n(i[1],prec=15)) for i in secondSlopeFunctionPoints],header_row=['$x$',"$y=f''(x)$"],frame=True, align='right')))
            except Exception as errmsg:
                tempTable = ([(n(i[0],prec=6),n(i[1],prec=15)) for i in secondSlopeFunctionPoints])
                tempTable.insert(0,['$x$',"$y=f''(x)$"])
                html.table(rows=tempTable,header=True)
    elif level == 'continuity':
        domMin, domMax = xrange
        segmentsGraph = []
        Graph = plot(0)
        for i in range(5,30):
            color = 'red'
            functionPoints = findFunctionPoints(f, filter=filter, steps=i)
            segmentGraph = line(functionPoints,xmin=domMin,xmax=domMax,ymin=ymin,ymax=ymax,color='red',alpha=1,thickness=2)
            segmentsGraph.append(segmentGraph)
        for i in range(len(functionPoints)):
            if i >= len(functionPoints)-1:
                break
            else:
                firstPoint = functionPoints[i]
                lastPoint = functionPoints[i+1]
            if firstPoint[1]<ymin or firstPoint[1]>ymax:
                continue
            slope = (firstPoint[1]-lastPoint[1])/(firstPoint[0]-lastPoint[0])
            g(x) = slope*(x-firstPoint[0]) + firstPoint[1]
            boldGraph = plot(g(x),xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax,color='slateblue',alpha=1,thickness=3)
            segmentsGraph.append(segmentGraph+boldGraph)
        show(animate(segmentsGraph,figsize=[10,6],xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax,gridlines='major',gridlinesstyle=dict(color="slategray", linestyle=":")),delay=500/deltaX)
        if logAnis:
            a=animate(segmentsGraph,figsize=[10,6],xmin=domMin-xStep,xmax=domMax+xStep,ymin=ymin,ymax=ymax,gridlines='major',gridlinesstyle=dict(color="slategray", linestyle=":"))
            a.gif(savefile = 'graph log files/'+fName+str(fileCount)+'.gif',delay=50,iterations=2)
            fileCount += 1
︡b1d7ee11-c923-497e-a27d-13a580773dce︡{"stderr":"/cocalc/lib/python2.7/site-packages/smc_sagews/sage_server.py:1234: DeprecationWarning: Substitution using function-call syntax and unnamed arguments is deprecated and will be removed from a future release of Sage; you can use named arguments instead, like EXPR(x=..., y=...)\nSee http://trac.sagemath.org/5930 for details.\n  flags=compile_flags), namespace, locals)\n"}︡{"interact":{"controls":[{"button_classes":null,"buttons":false,"control_type":"selector","default":0,"label":"function","lbls":["(x + 4)*(x - 5)","-(x + 5)*(x + 1)*(x - 5)","(x + 5)*(x + 2)*(x - 1)*(x - 4)","1/x","1/(x - 3) + 4","sqrt(x - 3) + 2","e^x","e^(x + 2) - 3","log(x)","log(x + 3)","-e^(-x) + 1"],"ncols":null,"nrows":null,"var":"f","width":50},{"animate":true,"control_type":"range-slider","default":[60,100],"display_value":true,"label":"xrange","vals":["-20.0000000000000","-19.7500000000000","-19.5000000000000","-19.2500000000000","-19.0000000000000","-18.7500000000000","-18.5000000000000","-18.2500000000000","-18.0000000000000","-17.7500000000000","-17.5000000000000","-17.2500000000000","-17.0000000000000","-16.7500000000000","-16.5000000000000","-16.2500000000000","-16.0000000000000","-15.7500000000000","-15.5000000000000","-15.2500000000000","-15.0000000000000","-14.7500000000000","-14.5000000000000","-14.2500000000000","-14.0000000000000","-13.7500000000000","-13.5000000000000","-13.2500000000000","-13.0000000000000","-12.7500000000000","-12.5000000000000","-12.2500000000000","-12.0000000000000","-11.7500000000000","-11.5000000000000","-11.2500000000000","-11.0000000000000","-10.7500000000000","-10.5000000000000","-10.2500000000000","-10.0000000000000","-9.75000000000000","-9.50000000000000","-9.25000000000000","-9.00000000000000","-8.75000000000000","-8.50000000000000","-8.25000000000000","-8.00000000000000","-7.75000000000000","-7.50000000000000","-7.25000000000000","-7.00000000000000","-6.75000000000000","-6.50000000000000","-6.25000000000000","-6.00000000000000","-5.75000000000000","-5.50000000000000","-5.25000000000000","-5.00000000000000","-4.75000000000000","-4.50000000000000","-4.25000000000000","-4.00000000000000","-3.75000000000000","-3.50000000000000","-3.25000000000000","-3.00000000000000","-2.75000000000000","-2.50000000000000","-2.25000000000000","-2.00000000000000","-1.75000000000000","-1.50000000000000","-1.25000000000000","-1.00000000000000","-0.750000000000000","-0.500000000000000","-0.250000000000000","0.000000000000000","0.250000000000000","0.500000000000000","0.750000000000000","1.00000000000000","1.25000000000000","1.50000000000000","1.75000000000000","2.00000000000000","2.25000000000000","2.50000000000000","2.75000000000000","3.00000000000000","3.25000000000000","3.50000000000000","3.75000000000000","4.00000000000000","4.25000000000000","4.50000000000000","4.75000000000000","5.00000000000000","5.25000000000000","5.50000000000000","5.75000000000000","6.00000000000000","6.25000000000000","6.50000000000000","6.75000000000000","7.00000000000000","7.25000000000000","7.50000000000000","7.75000000000000","8.00000000000000","8.25000000000000","8.50000000000000","8.75000000000000","9.00000000000000","9.25000000000000","9.50000000000000","9.75000000000000","10.0000000000000","10.2500000000000","10.5000000000000","10.7500000000000","11.0000000000000","11.2500000000000","11.5000000000000","11.7500000000000","12.0000000000000","12.2500000000000","12.5000000000000","12.7500000000000","13.0000000000000","13.2500000000000","13.5000000000000","13.7500000000000","14.0000000000000","14.2500000000000","14.5000000000000","14.7500000000000","15.0000000000000","15.2500000000000","15.5000000000000","15.7500000000000","16.0000000000000","16.2500000000000","16.5000000000000","16.7500000000000","17.0000000000000","17.2500000000000","17.5000000000000","17.7500000000000","18.0000000000000","18.2500000000000","18.5000000000000","18.7500000000000","19.0000000000000","19.2500000000000","19.5000000000000","19.7500000000000","20.0000000000000"],"var":"xrange","width":200},{"animate":true,"control_type":"range-slider","default":[35,50],"display_value":true,"label":"yrange","vals":["-400","-390","-380","-370","-360","-350","-340","-330","-320","-310","-300","-290","-280","-270","-260","-250","-240","-230","-220","-210","-200","-190","-180","-170","-160","-150","-140","-130","-120","-110","-100","-90","-80","-70","-60","-50","-40","-30","-20","-10","0","10","20","30","40","50","60","70","80","90","100","110","120","130","140","150","160","170","180","190","200","210","220","230","240","250","260","270","280","290","300","310","320","330","340","350","360","370","380","390","400","410","420","430","440","450","460","470","480","490","500"],"var":"yrange","width":200},{"button_classes":null,"buttons":true,"control_type":"selector","default":0,"label":"type","lbls":["table","points","lines","segments","plot"],"ncols":null,"nrows":null,"var":"type","width":20},{"button_classes":null,"buttons":true,"control_type":"selector","default":0,"label":"level","lbls":["function","derivative","2nd derivative","continuity"],"ncols":null,"nrows":null,"var":"level","width":20},{"button_classes":null,"buttons":false,"control_type":"selector","default":0,"label":"tables","lbls":["function","derivative","2nd derivative"],"ncols":null,"nrows":null,"var":"tables","width":20},{"control_type":"checkbox","default":false,"label":"Show Slopes","readonly":false,"var":"showSlope"},{"control_type":"checkbox","default":true,"label":"Show Animation","readonly":false,"var":"animateGr"},{"control_type":"checkbox","default":true,"label":"Log Animations","readonly":false,"var":"logAnis"},{"button_classes":null,"buttons":true,"control_type":"selector","default":0,"label":"filter","lbls":["normal","negative","absolute value","square","inverse"],"ncols":null,"nrows":null,"var":"filter","width":20},{"animate":true,"control_type":"slider","default":70,"display_value":true,"label":"shift","vals":["-50","-49","-48","-47","-46","-45","-44","-43","-42","-41","-40","-39","-38","-37","-36","-35","-34","-33","-32","-31","-30","-29","-28","-27","-26","-25","-24","-23","-22","-21","-20","-19","-18","-17","-16","-15","-14","-13","-12","-11","-10","-9","-8","-7","-6","-5","-4","-3","-2","-1","0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30"],"var":"shiftText","width":20},{"animate":true,"control_type":"slider","default":99,"display_value":true,"label":"aspect","vals":["1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32","33","34","35","36","37","38","39","40","41","42","43","44","45","46","47","48","49","50","51","52","53","54","55","56","57","58","59","60","61","62","63","64","65","66","67","68","69","70","71","72","73","74","75","76","77","78","79","80","81","82","83","84","85","86","87","88","89","90","91","92","93","94","95","96","97","98","99","100","101","102","103","104","105","106","107","108","109","110","111","112","113","114","115","116","117","118","119","120","121","122","123","124","125","126","127","128","129","130","131","132","133","134","135","136","137","138","139","140","141","142","143","144","145","146","147","148","149","150","151","152","153","154","155","156","157","158","159","160","161","162","163","164","165","166","167","168","169","170","171","172","173","174","175","176","177","178","179","180","181","182","183","184","185","186","187","188","189","190","191","192","193","194","195","196","197","198","199","200","201","202","203","204","205","206","207","208","209","210","211","212","213","214","215","216","217","218","219","220","221","222","223","224","225","226","227","228","229","230","231","232","233","234","235","236","237","238","239","240","241","242","243","244","245","246","247","248","249","250","251","252","253","254","255","256","257","258","259","260","261","262","263","264","265","266","267","268","269","270","271","272","273","274","275","276","277","278","279","280","281","282","283","284","285","286","287","288","289","290","291","292","293","294","295","296","297","298","299","300"],"var":"aspect","width":20},{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"deltaX","vals":["5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32","33","34","35","36","37","38","39","40","41","42","43","44","45","46","47","48","49","50","51","52","53","54","55","56","57","58","59","60","61","62","63","64","65","66","67","68","69","70","71","72","73","74","75","76","77","78","79","80","81","82","83","84","85","86","87","88","89","90","91","92","93","94","95","96","97","98","99","100"],"var":"deltaX","width":20},{"control_type":"input-box","default":"general quartic","label":"Log File Name","nrows":1,"readonly":false,"submit_button":null,"type":"<type 'str'>","var":"fName","width":null},{"classes":null,"control_type":"button","default":"Update","icon":null,"label":null,"var":"auto_update","width":null}],"flicker":false,"id":"0b4830dd-cdb7-47c4-8ae3-5bc0347bec42","layout":[[["f",12,null]],[["xrange",12,null]],[["yrange",12,null]],[["type",12,null]],[["level",12,null]],[["tables",12,null]],[["showSlope",12,null]],[["animateGr",12,null]],[["logAnis",12,null]],[["filter",12,null]],[["shiftText",12,null]],[["aspect",12,null]],[["deltaX",12,null]],[["fName",12,null]],[["",12,null]],[["auto_update",2]]],"style":"None","width":200}}︡{"done":true}{"done":true}︡
︠f4c6c502-ee94-498f-9d29-793ec7f16f48s︠
@interact
def Midpoint(ShowSolution=checkbox(default=True, label='Check to show solution')):
    x1=Integer(randint(-5,10))
    x2=Integer(randint(-5,10))
    y1=Integer(randint(-5,10))
    y2=Integer(randint(-5,10))
    html('Find the midpoint of $(%s,%s)$ and $(%s,%s)$.'% (x1,y1,x2,y2))
    if ShowSolution==True:
        html('$M=\\left(\\frac{(%s)+(%s)}{2},\\frac{(%s)+(%s)}{2}\\right)$'%(x1,x2,y1,y2))
        html('$M=\\left(\\frac{%s}{2},\\frac{%s}{2}\\right)$'%(x1+x2,y1+y2))
        if (x1+x2)/2 in ZZ or(x1+x2)/2 in ZZ:
            html('$M=\\left(%s,%s\\right)$'%(latex((x1+x2)/2),latex((y1+y2)/2)))
︡b5baa3d7-c8e6-4558-bfee-c609b65d6075︡{"interact":{"controls":[{"control_type":"checkbox","default":true,"label":"Check to show solution","readonly":false,"var":"ShowSolution"}],"flicker":false,"id":"ba4bb7b8-5a31-45d3-b279-460eaaab3b68","layout":[[["ShowSolution",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}
︠20757bde-5f04-438a-968c-64506b042748s︠
f=(x-2)*(x+2)*x
pretty_print(f.expand())
︡ec06b200-9084-4f04-b651-37998c6b78e6︡{"html":"<div align='center'>$\\displaystyle x^{3} - 4 \\, x$</div>"}︡{"done":true}









